# CyberChef server

[![](https://img.shields.io/badge/license-Apache%202.0-blue.svg)](https://github.com/gchq/CyberChef-server/blob/master/LICENSE)
[![Gitter](https://badges.gitter.im/gchq/CyberChef.svg)](https://gitter.im/gchq/CyberChef?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge)


Run CyberChef in a server and provide an API for clients to send [Cyberchef](https://gchq.github.io/CyberChef/) recipes to bake.

## Motivation

CyberChef has a useful Node.js API, but sometimes we want to be able to programmatically run CyberChef recipes in languages other than JavaScript. By running this server, you can use CyberChef operations in any language, as long as you can communicate via HTTP.

## Example use
Assuming you've downloaded the repository and are running it locally:
```bash
curl -X POST -H "Content-Type:application/json" -d '{"input":"... ---:.-.. --- -. --. --..--:.- -. -..:- .... .- -. -.- ...:..-. --- .-.:.- .-.. .-..:- .... .:..-. .. ... ....", "recipe":{"op":"from morse code", "args": {"wordDelimiter": "Colon"}}}' localhost:3000/bake
```
response:
```
{
    value: "SO LONG, AND THANKS FOR ALL THE FISH",
    type: "string"
}
```


## Features
- **Compatible with recipes saved from CyberChef**.
After using [CyberChef](https://gchq.github.io/CyberChef/) to experiment and find a suitable recipe, the exported recipe JSON can be used to post to the `/bake` endpoint. Just copy/paste it in as your `recipe` property as part of the POST body.


## Installing
- Clone the repository
- `cd` into the project and run `npm install`
- Run `npm run`
- In a browser, navigate to `localhost:3000` to see usage documentation.



## API overview
> For full documentation of the API, you can find the swagger page hosted at the root url. See [Installing](#Installing) to run the application and browse the docs.

Currently the server just has one endpoint: `/bake`. This endpoint accepts a POST request with the following body:

|Parameter|Type|Description|
|---|---|---|
input|String|The input data for the recipe. Currently accepts strings.
recipe|String or Object or Array|One or more operations, with optional arguments. Uses default arguments if they're not defined here.
outputType (optional)|String|The [Data Type](https://github.com/gchq/CyberChef/wiki/Adding-a-new-operation#data-types) that you would like the result of the bake to be returned as. This will not work with `File` or `List<File>` at the moment.

#### Example: one operation, default arguments
```javascript
{
    "input": "One, two, three, four.",
    "recipe": "to decimal"
}
```

Response:
```javascript
{
    value: "79 110 101 44 32 116 119 111 44 32 116 104 114 101 101 44 32 102 111 117 114 46",
    type: "string"
}

```
> For more information on how operation names are handled, see the [Node API docs](https://github.com/gchq/CyberChef/wiki/Node-API#operation-names)


#### Example: one operation, non-default arguments by name
```javascript
{
    "input": "One, two, three, four.",
    "recipe": {
        "op": "to decimal",
        "args": {
            "delimiter": "Colon"
        }
    }
}
```
Response:
```javascript
{
    value: "79:110:101:44:32:116:119:111:44:32:116:104:114:101:101:44:32:102:111:117:114:46",
    type: "string"
}
```

#### Example: one operation, non-default arguments by position
```javascript
{
    "input": "One, two, three, four.",
    "recipe": {
        "op": "to decimal",
        "args": ["Colon"]
    }
}
```
Response:
```javascript
{
    value: "79:110:101:44:32:116:119:111:44:32:116:104:114:101:101:44:32:102:111:117:114:46",
    type: "string"
}
```

#### Example: all together
```javascript
{
    "input": "One, two, three, four.",
    "recipe": [
        {
            "op":"to decimal",
            "args": {
                "delimiter": "CRLF"
            }
        },
        {
            "op": "swap endianness",
            "args": ["Raw"]
        },
        "MD4"
    ]
}

```
Response:
```javascript
{
    value: "31d6cfe0d16ae931b73c59d7e0c089c0",
    type: "string"
}
```


#### Example: Define outputType
`toDecimal` has an outputType of `string`. Here we are asking to translate the output to a number before returning.
```javascript
{
    "input": "One, two, three, four.",
    "recipe": "to decimal",
    "outputType": "number"
}
```
Response:
```javascript
{
    // Be wary, type conversions do not always behave as expected.
    "value": 79,
    "type": "number"
}
```

## Licencing

CyberChef-server is released under the [Apache 2.0 Licence](https://www.apache.org/licenses/LICENSE-2.0) and is covered by [Crown Copyright](https://www.nationalarchives.gov.uk/information-management/re-using-public-sector-information/copyright-and-re-use/crown-copyright/).